/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_DEBUG_H_
#define __GDAM_DEBUG_H_
#include <glib.h>

extern gboolean      gdam_debugging_on;

void                 gdam_debug(const char* format, ...);


/* Do not define this-- it doesn't actually work for some reason... */
//#define MEMORY_DEBUGGING

#ifdef MEMORY_DEBUGGING
#include <stdlib.h>
void*  g_malloc_debug(size_t, int line, const char* filename);
void*  g_realloc_debug(void*, size_t, int line, const char* filename);
void*  g_malloc0_debug(size_t, int line, const char* filename);
void   g_free_debug(void*, int line, const char* filename);

#define g_malloc(size)		g_malloc_debug(size, __LINE__, __FILE__)
#define g_realloc(ptr, size)	g_realloc_debug(ptr, size, __LINE__, __FILE__)
#define g_malloc0(size)		g_malloc0_debug(size, __LINE__, __FILE__)
#define g_free(ptr)		g_free_debug(ptr, __LINE__, __FILE__)

#endif

#if GDAM_DEBUG_REFERENCING

#include <gtk/gtkobject.h>
#define gtk_object_ref(obj) 						    \
	G_STMT_START {							    \
		g_message("ref: %p, line %d, file %s; "			    \
				"type=%s; count=%d++",			    \
					obj, __LINE__,__FILE__,		    \
					gtk_type_name(GTK_OBJECT_TYPE(obj)),\
				(obj)->ref_count);			    \
				gtk_object_ref(obj);			    \
		       } G_STMT_END
#define gtk_object_unref(obj)                                               \
        G_STMT_START {							    \
		g_message("unref: %p, line %d, file %s; "		    \
		                  "type=%s; count=%d--",                    \
					obj, __LINE__,__FILE__,
					gtk_type_name(GTK_OBJECT_TYPE(obj)),\
					(obj)->ref_count);		    \
				gtk_object_unref(obj);			    \
			       } G_STMT_END
#endif

#endif
