/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_BENDER_H_
#define __GDAM_BENDER_H_


typedef struct _GdamBender GdamBender;
typedef struct _GdamBenderClass GdamBenderClass;
typedef struct _GdamBenderChange GdamBenderChange;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <gdam/gdambufferingfilter.h>
typedef enum _GdamBenderChangeMode GdamBenderChangeMode;
enum _GdamBenderChangeMode {
	/* When the changes are over, keep playing from that point. */
	GDAM_BENDER_CHANGE_CONTINUOUS,

	/* Keep a mark that can be jumped to ("jump_to_mark")
	 * and set ("set_mark"). */
	GDAM_BENDER_CHANGE_KEEP_TIME,

	/* Like "keep_time" but after changes are over,
	 * snap to the mark. */
	GDAM_BENDER_CHANGE_SNAP_TIME,

	/* When the changes are over, return to where we were
	 * when the changes started. */
	GDAM_BENDER_CHANGE_RETURN

};

GtkType gdam_bender_get_type();
#define GDAM_TYPE_BENDER			(gdam_bender_get_type ())
#define GDAM_BENDER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_BENDER, GdamBender))
#define GDAM_BENDER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_BENDER, GdamBenderClass))
#define GDAM_IS_BENDER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_BENDER))
#define GDAM_IS_BENDER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_BENDER))

struct _GdamBenderChange {
	int				num_samples;
	GdamFixed			new_rate;
	GdamBenderChange*		next;
};

struct _GdamBenderClass {
	GdamBufferingFilterClass	buffering_filter_class;
};

struct _GdamBender {
	GdamBufferingFilter		buffering_filter;

	/* List of change points. */
	GdamBenderChange*		changes;

	GdamBenderChangeMode		change_mode;

	/* In b16, just like GdamFixed;
	 * it may be outside of the buffers range. */
	gint64				backbuffer_offset;

	/* The position we started from if it was set with
	 * `changes_temporary'. */
	gint64				end_offset;

	/* For fading... */
	gint				fade_window;
	gint				fade_remaining;
	GdamSample			fade_sample;
	GdamSample			last_sample;

	/* Number of samples further than current position to
	 * read from subsource. */
	int				readahead;

	/* Base resample rate. */
	GdamFixed			rate;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
