/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsynthtwoiir.h"
#include <gdam/gdamplugin.h>

static int gdam_synth_two_iir_synthesize(GdamSynthSource* source,
                                         gdouble*         buf,
					 int              num)
{
	GdamSynthTwoIir* two_iir = GDAM_SYNTH_TWO_IIR(source);
	GdamSynthSource* filtered = ((GdamSynthFilter*)two_iir)->filtered;
	double coeff0 = two_iir->coeff[0];
	double coeff1 = two_iir->coeff[1];
	double coeff2 = two_iir->coeff[2];
	double last0 = two_iir->last_samples[0];
	double last1 = two_iir->last_samples[1];
	int num_read = gdam_synth_source_synthesize(filtered, buf, num);
	num = num_read;
	while (num_read-- > 0) {
		gdouble output;
		gdouble cur = *buf;
		output = coeff2 * last0
		       + coeff1 * last1
		       + coeff0 * cur;
		last0 = last1;
		last1 = output;
		*buf++ = output;
	}
	two_iir->last_samples[0] = last0;
	two_iir->last_samples[1] = last1;
	return num;
}

static void gdam_synth_two_iir_init(GdamSynthTwoIir* synth_two_iir)
{
}
static void gdam_synth_two_iir_class_init(GdamSynthSourceClass* klass)
{
	klass->synthesize = gdam_synth_two_iir_synthesize;
}

GtkType gdam_synth_two_iir_get_type()
{
	static GtkType synth_two_iir_type = 0;
	if (!synth_two_iir_type) {
		GtkType parent_type;
		static const GtkTypeInfo synth_two_iir_info = {
			"GdamSynthTwoIir",
			sizeof(GdamSynthTwoIir),
			sizeof(GdamSynthTwoIirClass),
			(GtkClassInitFunc) gdam_synth_two_iir_class_init,
			(GtkObjectInitFunc) gdam_synth_two_iir_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		parent_type = gdam_plugin_get_type_by_name("server",
		                                           "GdamSynthFilter");
		synth_two_iir_type = gtk_type_unique(parent_type,
		                                 &synth_two_iir_info);
	}
	return synth_two_iir_type;
}
